@@ -10,7 +10,7 @@ fn main() -> Result<(), Box<dyn Error>> {
     info!("start");
 
     let config = Config::from_args();
-    let (send, recv) = sync_channel(100);
+    let (send, recv) = sync_channel(config.channels_size.into());
     let handle = blocks_iterator::iterate(config, send);
     while let Some(block_extra) = recv.recv()? {
         log!(
