@@ -28,10 +28,10 @@ fn main() -> Result<(), Box<dyn Error>> {
     info!("start");
 
     let config = Config::from_args();
-    let (send, recv) = sync_channel(100);
-    let handle = blocks_iterator::iterate(config, send);
+    let (send, recv) = sync_channel(config.channels_size.into());
+    let (send_script, recv_script) = sync_channel(config.channels_size.into());
 
-    let (send_script, recv_script) = sync_channel(BATCH);
+    let handle = blocks_iterator::iterate(config, send);
 
     let process_handle = thread::spawn(move || {
         let error_count = AtomicUsize::new(0);
