@@ -12,12 +12,14 @@
 #![deny(unused_mut)]
 #![deny(dead_code)]
 #![deny(unused_imports)]
-#![deny(missing_docs)]
+//#![deny(missing_docs)]
 #![deny(unused_must_use)]
 
-use bitcoin::{Block, BlockHash, OutPoint, Transaction, TxOut, Txid};
+use bitcoin::{Block, BlockHash, OutPoint, Transaction, TxOut};
 use log::{info, Level};
-use std::collections::{HashMap, HashSet};
+use std::collections::HashMap;
+use std::convert::TryFrom;
+use std::ops::DerefMut;
 use std::path::PathBuf;
 use std::sync::mpsc::{sync_channel, SyncSender};
 use std::thread;
@@ -26,16 +28,19 @@ use std::time::Instant;
 use structopt::StructOpt;
 
 mod fee;
-mod parse;
-mod read;
+mod read_detect;
 mod reorder;
 mod truncmap;
 
 // re-exporting deps
 pub use bitcoin;
+use bitcoin::consensus::Decodable;
 pub use fxhash;
 pub use glob;
 pub use log;
+use std::fs::File;
+use std::io::{BufReader, Seek, SeekFrom};
+use std::sync::{Arc, Mutex};
 pub use structopt;
 
 /// Configuration parameters, most important the bitcoin blocks directory
@@ -59,6 +64,10 @@ pub struct Config {
     /// When parsing testnet blocks, it may be necessary to increase this a lot
     #[structopt(short, long, default_value = "6")]
     pub max_reorg: u8,
+
+    /// Size of the channels used to pass messages between threads
+    #[structopt(short, long, default_value = "0")]
+    pub channels_size: u8,
 }
 
 /// The bitcoin block and additional metadata returned by the [iterate] method
@@ -79,8 +88,40 @@ pub struct BlockExtra {
     /// Note that when configuration `skip_script_pubkey` is true, the script is empty,
     /// when `skip_prevout` is true, this map is empty.
     pub outpoint_values: HashMap<OutPoint, TxOut>,
-    /// Precomputed set of txid present in `block`
-    pub tx_hashes: HashSet<Txid>,
+}
+
+/// Before reorder we keep only the position of the block in the file system and data relative
+/// to the block hash, the previous hash and the following hash (populated during reorder phase)
+/// We will need to read the block from disk again, but by doing so we will avoid using too much
+/// memory in the [`OutOfOrderBlocks`] map.
+#[derive(Debug)]
+pub struct FsBlock {
+    pub file: Arc<Mutex<File>>,
+    pub start: usize,
+    pub end: usize,
+    pub hash: BlockHash,
+    pub prev: BlockHash,
+    pub next: Vec<BlockHash>,
+}
+
+impl TryFrom<FsBlock> for BlockExtra {
+    type Error = ();
+
+    fn try_from(fs_block: FsBlock) -> Result<Self, Self::Error> {
+        let mut guard = fs_block.file.lock().unwrap();
+        let file = guard.deref_mut();
+        file.seek(SeekFrom::Start(fs_block.start as u64))
+            .map_err(|_| ())?;
+        let reader = BufReader::new(file);
+        Ok(BlockExtra {
+            block: Block::consensus_decode(reader).map_err(|_| ())?,
+            block_hash: fs_block.hash,
+            size: (fs_block.end - fs_block.start) as u32,
+            next: fs_block.next,
+            height: 0,
+            outpoint_values: Default::default(),
+        })
+    }
 }
 
 impl BlockExtra {
@@ -111,26 +152,22 @@ impl BlockExtra {
 
 /// Read `blocks*.dat` contained in the `config.blocks_dir` directory and returns [BlockExtra]
 /// through a channel supplied from the caller. Blocks returned are ordered from the genesis to the
-/// highest block in the dircetory (minus `config.max_reorg`).
+/// highest block in the directory (minus `config.max_reorg`).
 /// In this call threads are spawned, caller must call [std::thread::JoinHandle::join] on the returning handle.
 pub fn iterate(config: Config, channel: SyncSender<Option<BlockExtra>>) -> JoinHandle<()> {
     thread::spawn(move || {
         let now = Instant::now();
 
-        let (send_blobs, receive_blobs) = sync_channel(2);
-
-        let mut read = read::Read::new(config.blocks_dir.clone(), send_blobs);
+        // FsBlock is a small struct (~120b), so 10_000 is not a problem but allows the read_detect to read ahead the next block file
+        let (send_block_fs, receive_block_fs) = sync_channel(0);
+        let mut read =
+            read_detect::ReadDetect::new(config.blocks_dir.clone(), config.network, send_block_fs);
         let read_handle = thread::spawn(move || {
             read.start();
         });
 
-        let (send_blocks, receive_blocks) = sync_channel(200);
-        let mut parse = parse::Parse::new(config.network, receive_blobs, send_blocks);
-        let parse_handle = thread::spawn(move || {
-            parse.start();
-        });
-
-        let (send_ordered_blocks, receive_ordered_blocks) = sync_channel(200);
+        let (send_ordered_blocks, receive_ordered_blocks) =
+            sync_channel(config.channels_size.into());
         let send_ordered_blocks = if config.skip_prevout {
             // if skip_prevout is true, we send directly to end step
             channel.clone()
@@ -140,7 +177,7 @@ pub fn iterate(config: Config, channel: SyncSender<Option<BlockExtra>>) -> JoinH
         let mut reorder = reorder::Reorder::new(
             config.network,
             config.max_reorg,
-            receive_blocks,
+            receive_block_fs,
             send_ordered_blocks,
         );
         let orderer_handle = thread::spawn(move || {
@@ -156,7 +193,6 @@ pub fn iterate(config: Config, channel: SyncSender<Option<BlockExtra>>) -> JoinH
         }
 
         read_handle.join().unwrap();
-        parse_handle.join().unwrap();
         orderer_handle.join().unwrap();
         info!("Total time elapsed: {}s", now.elapsed().as_secs());
     })
