@@ -1,14 +1,15 @@
-use crate::{periodic_log_level, BlockExtra};
+use crate::{periodic_log_level, BlockExtra, FsBlock};
 use bitcoin::blockdata::constants::genesis_block;
 use bitcoin::{BlockHash, Network};
 use log::{info, log, warn};
 use std::collections::HashMap;
+use std::convert::TryInto;
 use std::sync::mpsc::Receiver;
 use std::sync::mpsc::SyncSender;
 use std::time::Instant;
 
 pub struct Reorder {
-    receiver: Receiver<Option<BlockExtra>>,
+    receiver: Receiver<Option<Vec<FsBlock>>>,
     sender: SyncSender<Option<BlockExtra>>,
     height: u32,
     next: BlockHash,
@@ -16,7 +17,7 @@ pub struct Reorder {
 }
 
 struct OutOfOrderBlocks {
-    blocks: HashMap<BlockHash, BlockExtra>,
+    blocks: HashMap<BlockHash, FsBlock>,
     follows: HashMap<BlockHash, Vec<BlockHash>>,
     max_reorg: u8,
 }
@@ -30,24 +31,24 @@ impl OutOfOrderBlocks {
         }
     }
 
-    fn add(&mut self, mut block_extra: BlockExtra) {
-        let prev_hash = block_extra.block.header.prev_blockhash;
+    fn add(&mut self, mut raw_block: FsBlock) {
+        let prev_hash = raw_block.prev;
         self.follows
             .entry(prev_hash)
-            .and_modify(|e| e.push(block_extra.block_hash))
-            .or_insert_with(|| vec![block_extra.block_hash]);
+            .and_modify(|e| e.push(raw_block.hash))
+            .or_insert_with(|| vec![raw_block.hash]);
 
-        if let Some(follows) = self.follows.remove(&block_extra.block_hash) {
+        if let Some(follows) = self.follows.remove(&raw_block.hash) {
             for el in follows {
-                block_extra.next.push(el);
+                raw_block.next.push(el);
             }
         }
 
         if let Some(prev_block) = self.blocks.get_mut(&prev_hash) {
-            prev_block.next.push(block_extra.block_hash);
+            prev_block.next.push(raw_block.hash);
         }
 
-        self.blocks.insert(block_extra.block_hash, block_extra);
+        self.blocks.insert(raw_block.hash, raw_block);
     }
 
     /// check the block identified by `hash` has at least `self.max_reorgs` blocks after, to be sure it's not a reorged block
@@ -68,14 +69,11 @@ impl OutOfOrderBlocks {
         None
     }
 
-    fn remove(&mut self, hash: &BlockHash) -> Option<BlockExtra> {
+    fn remove(&mut self, hash: &BlockHash) -> Option<FsBlock> {
         if let Some(next) = self.exist_and_has_followers(hash, vec![]) {
             let mut value = self.blocks.remove(hash).unwrap();
             if value.next.len() > 1 {
-                warn!(
-                    "at {} fork to {:?} took {}",
-                    value.block_hash, value.next, next
-                );
+                warn!("at {} fork to {:?} took {}", value.hash, value.next, next);
             }
             value.next = vec![next];
             Some(value)
@@ -89,7 +87,7 @@ impl Reorder {
     pub fn new(
         network: Network,
         max_reorg: u8,
-        receiver: Receiver<Option<BlockExtra>>,
+        receiver: Receiver<Option<Vec<FsBlock>>>,
         sender: SyncSender<Option<BlockExtra>>,
     ) -> Reorder {
         Reorder {
@@ -115,41 +113,49 @@ impl Reorder {
     pub fn start(&mut self) {
         let mut busy_time = 0u128;
         let mut count = 0u32;
-        let mut now;
+        let mut now = Instant::now();
+        let mut last_height = 0;
         loop {
+            busy_time += now.elapsed().as_nanos();
             let received = self.receiver.recv().expect("cannot receive blob");
             now = Instant::now();
             match received {
-                Some(block_extra) => {
-                    log!(
-                        periodic_log_level(count),
-                        "reorder receive:{} size:{} follows:{} height:{} next:{}",
-                        block_extra.block_hash,
-                        self.blocks.blocks.len(),
-                        self.blocks.follows.len(),
-                        self.height,
-                        self.next
-                    );
+                Some(raw_blocks) => {
+                    for raw_block in raw_blocks {
+                        log!(
+                            periodic_log_level(count),
+                            "reorder receive:{} size:{} follows:{} next:{}",
+                            raw_block.hash,
+                            self.blocks.blocks.len(),
+                            self.blocks.follows.len(),
+                            self.next
+                        );
 
-                    count += 1;
+                        count += 1;
 
-                    if self.blocks.blocks.len() > 10_000 {
-                        for block in self.blocks.blocks.values() {
-                            println!("{} {:?}", block.block_hash, block.next);
+                        // even tough should be 1024 -> https://github.com/bitcoin/bitcoin/search?q=BLOCK_DOWNLOAD_WINDOW
+                        // in practice it needs to be greater
+                        let max_block_to_reorder = 10_000;
+                        if self.blocks.blocks.len() > max_block_to_reorder {
+                            for block in self.blocks.blocks.values() {
+                                println!("{} {:?}", block.hash, block.next);
+                            }
+                            println!("next: {}", self.next);
+                            panic!("Reorder map grow more than {}", max_block_to_reorder);
+                        }
+                        self.blocks.add(raw_block);
+                        while let Some(block_to_send) = self.blocks.remove(&self.next) {
+                            let block_extra: BlockExtra =
+                                block_to_send.try_into().expect("should find the file");
+                            busy_time += now.elapsed().as_nanos();
+                            self.send(block_extra);
+                            now = Instant::now();
+                            last_height = self.height;
                         }
-                        println!("next: {}", self.next);
-                        panic!();
-                    }
-                    self.blocks.add(block_extra);
-                    while let Some(block_to_send) = self.blocks.remove(&self.next) {
-                        busy_time += now.elapsed().as_nanos();
-                        self.send(block_to_send);
-                        now = Instant::now();
                     }
                 }
                 None => break,
             }
-            busy_time += now.elapsed().as_nanos();
         }
         info!(
             "ending reorder next:{} #elements:{} #follows:{}",
@@ -157,7 +163,11 @@ impl Reorder {
             self.blocks.blocks.len(),
             self.blocks.follows.len()
         );
-        info!("ending reorder, busy time: {}s", busy_time / 1_000_000_000);
+        info!(
+            "ending reorder, busy time: {}s, last height: {}",
+            busy_time / 1_000_000_000,
+            last_height
+        );
         self.sender.send(None).expect("reorder cannot send none");
     }
 }
